/* Copyright (c) 2017 timothyjager
   JBC-Soldering-Controller
   MIT License. See LICENSE file for details.

 * \brief   Handles user input in the JBC soldering station.
 * \author  Kaloyan Dimitrov  www.kae-systems.com
 * \date    13 mar 2019

*/

 
/******************************************************************************/
/*    Types                                                                   */
/******************************************************************************/
enum CL_btnId_t {
    BTN_POWER_ID = 0,
    BTN_CRADLE_ID,
    BTN_COUNT
};

enum CL_buttonPhases_t {
    BTN_PHASE_IDLE     = 0,
    BTN_PHASE_PRESSED  = 1,
    BTN_PHASE_HOLD     = 3,
    BTN_PHASE_RELEASED = 2
};

struct CL_status_t {
    uint32_t  timePointPutInCradle;
       float  lastSetTemperature;
     uint8_t  buttonPhases;
};

/******************************************************************************/
/*    Private global variables                                                */
/******************************************************************************/
static struct CL_status_t  CL_status;

/******************************************************************************/
/*    Private operations prototypes                                           */
/******************************************************************************/
static void  CL_SetTipTemperature(float  temperature);
static enum  CL_buttonPhases_t  CL_GetButtonPhase(enum CL_btnId_t  btn);
static LED_color_t  CL_TemperatureToColor(double  temperature, bool  isIdle);

/******************************************************************************/
/*    Public operations                                                       */
/******************************************************************************/
void CL_Init(void)
{
    bool  onCradle;
    
    CL_status.buttonPhases = 0;
    CL_RefreshButtonPhases();
    CL_RefreshButtonPhases();
    
    onCradle = (0 == fastDigitalRead( CRADLE_SENSOR ));
    
    // When starting with the iron on the cradle, heat up to 100C
    params.setpoint = onCradle ? 100.0 : 300.0;
    knob.write(onCradle ? 100 : 300);
    status.encoder_pos = onCradle ? 100 : 300;
    CL_status.lastSetTemperature = 300.0;
}

/*!
 * \brief  Produces button phases for all handled buttons.
 */
void  CL_RefreshButtonPhases(void)
{
    CL_status.buttonPhases <<= 1;
    CL_status.buttonPhases &= 0b10101010;
    
    if (0 == fastDigitalRead( ENC_BUTTON ))
    {
        CL_status.buttonPhases |= (1 << 0);
    }
    
    if (0 == fastDigitalRead( CRADLE_SENSOR ))
    {
        CL_status.buttonPhases |= (1 << 2);
    }
}

/*!
 * \brief  Services the power button.
 */
void  CL_ButtonPowerTask(void)
{
    switch (CL_GetButtonPhase( BTN_POWER_ID ))
    {
        case BTN_PHASE_PRESSED:
        {
            if (iron_active)
            {
                CL_TurnHeatingOff();
            }
            else
            {
                CL_TurnHeatingOn();
            }
            
            iron_active = (iron_active) ? false : true;
            break;
        }
        case BTN_PHASE_RELEASED:
        {
            break;
        }
        default: break;
    }
}

/*!
 * \brief  Services the cradle sensor.
 */
void  CL_ButtonCradleTask(bool*  pRefreshDisplay)
{
    switch (CL_GetButtonPhase( BTN_CRADLE_ID ))
    {
        case BTN_PHASE_PRESSED:
        {
            if (iron_active)
            {
                CL_status.timePointPutInCradle = millis();
                CL_status.lastSetTemperature = params.setpoint; // Remember for later (when the iron is lifted)
                CL_SetTipTemperature( 100.0 );
                
                *pRefreshDisplay = true;
            }
            break;
        }
        case BTN_PHASE_HOLD:
        {
            if (iron_active)
            {
                if ((millis() >= (CL_status.timePointPutInCradle + 300000UL)) &&
                    (0 < status.pid_setpoint))
                {
                    // Turn the iron off after being idle for 5 minutes
                    iron_active = false;
                    CL_TurnHeatingOff();
                    
                    *pRefreshDisplay = true;
                }
            }
            break;
        }
        case BTN_PHASE_RELEASED:
        {
            CL_SetTipTemperature( CL_status.lastSetTemperature );
            CL_TurnHeatingOn();
          
            if (false == iron_active)
            {
                iron_active = true;
                *pRefreshDisplay = true;
            }
            
            knob.write((int32_t)CL_status.lastSetTemperature);
            status.encoder_pos = (int16_t)CL_status.lastSetTemperature;
            break;
        }
        default: break;
    }
}

/*!
 * \brief  Services the rotary encoder.
 */
void  CL_RotaryEncoderTask(bool*  pPositionChanged)
{
    static int16_t knob_pos_last = 0;

    status.encoder_pos = knob.read(); //>> 2;  //divide by 2 to decrease sensitivity

    //Check if knob has changed
    if (status.encoder_pos != knob_pos_last)
    {
        //Dont allow it to go negative.
        if (status.encoder_pos < 0)
        {
            knob.write(0);
            status.encoder_pos = 0;
        }
        else
        {    
            if (status.encoder_pos > upper_set_limit)
            {
                knob.write(upper_set_limit);
                status.encoder_pos = upper_set_limit;
            }
        }
        
        CL_SetTipTemperature((float)status.encoder_pos);
        knob_pos_last = status.encoder_pos;    
        *pPositionChanged = true;
    }
}

/*!
 * \brief  TODO
 */
void  CL_TipTemperatureTrackingTask(void)
{
    enum CL_buttonPhases_t  btnCradlePhase = CL_GetButtonPhase( BTN_CRADLE_ID );
               LED_color_t  colorLED0;
               LED_color_t  colorLED1;
                      bool  isIdle;
    
    isIdle = ((BTN_PHASE_PRESSED == btnCradlePhase) ||
              (BTN_PHASE_HOLD    == btnCradlePhase));
 
    colorLED0 = CL_TemperatureToColor(status.tip_temperature_c, isIdle);
    colorLED1 = CL_TemperatureToColor(status.tip_temperature_c2, isIdle);
    
    LED_SetColor(LED_ID_0, colorLED0);
    LED_SetColor(LED_ID_1, colorLED1);
}

/*!
 * \brief  TODO
 */
void  CL_TurnHeatingOn(void)
{
    noInterrupts();
    
    params.pid_mode = AUTOMATIC;
    myPID.SetMode(params.pid_mode);
    myPID2.SetMode(params.pid_mode);
    status.pid_setpoint = params.setpoint;
    
    interrupts();
}

/*!
 * \brief  TODO
 */
void  CL_TurnHeatingOff(void)
{
    noInterrupts();
    
    params.pid_mode = MANUAL;
    myPID.SetMode(params.pid_mode);
    myPID2.SetMode(params.pid_mode);
    status.pid_output = 0;
    status.pid_output2 = 0;
    status.pid_setpoint = 0;
    
    interrupts();
}

/******************************************************************************/
/*    Private operations                                                      */
/******************************************************************************/
static void  CL_SetTipTemperature(float  temperature)
{
    params.setpoint = temperature;
    
    if (AUTOMATIC == myPID.GetMode())
    {
        status.pid_setpoint = (double)params.setpoint;
    }
}

/*!
 * \brief     Returns the current phase of requsted button.
 * \param[in] btn - button to query
 * \return    current phase of type 'enum CL_buttonPhases_t'
 */
static enum CL_buttonPhases_t  CL_GetButtonPhase(enum CL_btnId_t  btn)
{
    return (enum CL_buttonPhases_t)((CL_status.buttonPhases >> (btn * 2)) & 3);
}

/*!
 * \brief     Maps a temperature to color.
 * \param[in] temperature of the soldering tip
 * \return    color
 */
static LED_color_t  CL_TemperatureToColor(double  temperature, bool  isIdle)
{
    LED_color_t  color;
    
    if (temperature > 1000)
    {
        return LED_COLOR_OFF; // Probably detached termo sensor?
    }
    
    if (isIdle)
    {
        return LED_COLOR_GREEN;
    }
    
    if (status.pid_setpoint > 0)
    {
        // Heating ON
        if (temperature > (status.pid_setpoint - 10))
        {
            color = LED_COLOR_RED;
        }
        else
        {
            color = LED_COLOR_ORANGE;
        }
    }
    else
    {
        // Heating OFF
        if (temperature < coldTemp)
        {
            color = LED_COLOR_BLUE;
        }
        else
        {
            color = LED_COLOR_CYAN;
        }
    }
    
    return color;
}
